//modules/activiti-engine/src/main/java/org/activiti/engine/impl/test/PvmTestCase.java
/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.activiti.engine.impl.test;

import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

import org.activiti.engine.impl.util.ClassNameUtil;
import org.activiti.engine.impl.util.LogUtil;
import org.activiti.engine.impl.util.LogUtil.ThreadLogMode;


/**
 * @author Tom Baeyens
 */
public class PvmTestCase extends TestCase {

  protected static final String EMPTY_LINE = "                                                                                           ";

  static {
    LogUtil.readJavaUtilLoggingConfigFromClasspath();
  }

  private static Logger log = Logger.getLogger(PvmTestCase.class.getName());

  protected static final ThreadLogMode DEFAULT_THREAD_LOG_MODE = ThreadLogMode.INDENT;

  protected ThreadLogMode threadRenderingMode;
  protected boolean isEmptyLinesEnabled = true;

  public PvmTestCase() {
    this(DEFAULT_THREAD_LOG_MODE);
  }

  public PvmTestCase(ThreadLogMode threadRenderingMode) {
    this.threadRenderingMode = threadRenderingMode;
  }

  public void assertTextPresent(String expected, String actual) {
    if ( (actual==null)
         || (actual.indexOf(expected)==-1)
       ) {
      throw new AssertionFailedError("expected presence of ["+expected+"], but was ["+actual+"]");
    }
  }

  @Override
  protected void runTest() throws Throwable {
    LogUtil.resetThreadIndents();
    ThreadLogMode oldThreadRenderingMode = LogUtil.setThreadLogMode(threadRenderingMode);

    if (log.isLoggable(Level.FINE)) {
      if (isEmptyLinesEnabled) {
        log.fine(EMPTY_LINE);
      }
      log.fine("#### START "+ClassNameUtil.getClassNameWithoutPackage(this)+"."+getName()+" ###########################################################");
    }

    /*genesis generated change*/
    if (!((org.activiti.engine.impl.test.PvmTestCase.log) instanceof java.lang.Override))
        return ;
    try {
        super.runTest();
    } catch (junit.framework.AssertionFailedError e) {
        org.activiti.engine.impl.test.PvmTestCase.log.severe(org.activiti.engine.impl.test.PvmTestCase.EMPTY_LINE);
        org.activiti.engine.impl.test.PvmTestCase.log.log(java.util.logging.Level.SEVERE, ("ASSERTION FAILED: " + e), e);
        throw e;
    } catch (java.lang.Throwable e) {
        org.activiti.engine.impl.test.PvmTestCase.log.severe(org.activiti.engine.impl.test.PvmTestCase.EMPTY_LINE);
        org.activiti.engine.impl.test.PvmTestCase.log.log(java.util.logging.Level.SEVERE, ("EXCEPTION: " + e), e);
        throw e;
    } finally {
        org.activiti.engine.impl.test.PvmTestCase.log.fine((((("#### END " + (org.activiti.engine.impl.util.ClassNameUtil.getClassNameWithoutPackage(this))) + ".") + (getName())) + " #############################################################"));
        org.activiti.engine.impl.util.LogUtil.setThreadLogMode(oldThreadRenderingMode);
    }
  }

}
